﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class HW2_220B : MonoBehaviour
{
	Vector2 prevPos;
	ChuckMainInstance myChuck;

	// Use this for initialization
	void Start()
	{
		prevPos = ComputePos();
		myChuck = GetComponent<ChuckMainInstance>();
		RunChuck();

	}
	
	// Update is called once per frame
	void Update()
	{
		// compute movement
		Vector2 pos = ComputePos();
		float dist = ( pos - prevPos ).magnitude;

		// set movement
		myChuck.SetFloat( "mouseActivation", Mathf.Clamp( dist, 0, 0.1f ) );

		// remember
		prevPos = pos;
	}

	Vector2 ComputePos()
	{
		Vector3 mouse = Input.mousePosition;
		Vector2 mousePos = Vector2.zero;
		mousePos.x = mouse.x / Screen.width;
		mousePos.y = mouse.y / Screen.height;

		return mousePos;
	}

	void RunChuck()
	{
		myChuck.RunCode(@"
			global float mouseActivation;
			float theSumActivation;
			fun void ComputeActivation()
			{
				// leaky integrator
				0.996 => float activationReduction;
				
				while( true )
				{	
					// compute activation
					theSumActivation * activationReduction 
					    + mouseActivation * 0.2 => theSumActivation;
					
					if( theSumActivation > 1 ) { 1 => theSumActivation; }
					
					10::ms => now;
				}
			}
			spork ~ ComputeActivation();

			float detune;
			14 => int MAX_NOTES_TO_SAVE;
			13 => int NOTES_TO_PLAY;
			Event metro;
			Gain right => LPF rLPF => NRev rREV => dac.right;
			Gain left => LPF lLPF => NRev lREV => dac.left;
			0.1 => rREV.mix => lREV.mix;
			20000 => rLPF.freq => lLPF.freq;

			class AhhSynth extends Chubgraph
			{
				LiSa lisa => Gain myGain => outlet;
				
				// spawn rate: how often a new grain is spawned (ms)
				25 =>  float grainSpawnRateMS;
				0 =>  float grainSpawnRateVariationMS;
				0.0 =>  float grainSpawnRateVariationRateMS;
				
				// position: where in the file is a grain (0 to 1)
				0.61 =>  float grainPosition;
				0.2 =>  float grainPositionRandomness;
				
				// grain length: how long is a grain (ms)
				300 =>  float grainLengthMS;
				10 =>  float grainLengthRandomnessMS;
				
				// grain rate: how quickly is the grain scanning through the file
				1.004 =>  float grainRate; // 1.002 == in-tune Ab
				0.015 =>  float grainRateRandomness;
				
				// ramp up/down: how quickly we ramp up / down
				50 =>  float rampUpMS;
				200 =>  float rampDownMS;
				
				// gain: how loud is everything overall
				1 =>  float gainMultiplier;
				
				float myFreq;
				fun float freq( float f )
				{
					f => myFreq;
					61 => Std.mtof => float baseFreq;
					// 1.002 == in tune for 56 for aah4.wav
					// 1.004 == in tune for 60 for aah5.wav
					myFreq / baseFreq * 0.98 => grainRate;
					
					return myFreq;
				}
				
				fun float freq()
				{
					return myFreq;
				}
				
				fun float gain( float g )
				{
					g => myGain.gain;
					return g;
				}
				
				fun float gain()
				{
					return myGain.gain();
				}
				
				SndBuf buf; 
				me.dir() + ""aah5.wav"" => buf.read;
				buf.length() => lisa.duration;
				// copy samples in
				for( int i; i < buf.samples(); i++ )
				{
					lisa.valueAt( buf.valueAt( i ), i::samp );
				}
				
				buf.length() => dur bufferlen;
				
				// LiSa params
				100 => lisa.maxVoices;
				0.1 => lisa.gain;
				true => lisa.loop;
				false => lisa.record;
				
				// modulate
				SinOsc freqmod => blackhole;
				0.1 => freqmod.freq;
				
				0.1 => float maxGain;
				
				fun void SetGain()
				{
					while( true )
					{
						maxGain * gainMultiplier => lisa.gain;
						1::ms => now;
					}
				}
				spork ~ SetGain();
				
				
				fun void SpawnGrains()
				{
					// create grains
					while( true )
					{
						// grain length
						( grainLengthMS + Math.random2f( -grainLengthRandomnessMS / 2, grainLengthRandomnessMS / 2 ) )
						* 1::ms => dur grainLength;
						
						// grain rate
						grainRate + Math.random2f( -grainRateRandomness / 2, grainRateRandomness / 2 ) => float grainRate;
						
						// grain position
						( grainPosition + Math.random2f( -grainPositionRandomness / 2, grainPositionRandomness / 2 ) )
						* bufferlen => dur playPos;
						
						// grain: grainlen, rampup, rampdown, rate, playPos
						spork ~ PlayGrain( grainLength, rampUpMS::ms, rampDownMS::ms, grainRate, playPos);
						
						// advance time (time per grain)
						// PARAM: GRAIN SPAWN RATE
						grainSpawnRateMS::ms  + freqmod.last() * grainSpawnRateVariationMS::ms => now;
						grainSpawnRateVariationRateMS => freqmod.freq;
					}
				}
				spork ~ SpawnGrains();
				
				// sporkee
				fun void PlayGrain( dur grainlen, dur rampup, dur rampdown, float rate, dur playPos )
				{
					lisa.getVoice() => int newvoice;
					
					if(newvoice > -1)
					{
						lisa.rate( newvoice, rate );
						lisa.playPos( newvoice, playPos );
						lisa.rampUp( newvoice, rampup );
						( grainlen - ( rampup + rampdown ) ) => now;
						lisa.rampDown( newvoice, rampdown) ;
						rampdown => now;
					}
				}   
			}


			class TanDistortion extends Chugen
			{
				fun float tick( float in )
				{
					in * in => float square;
					in * square => float cubic;
					cubic * square => float five;
					five * square => float seven;
					return 0.5 * (in - cubic / 3 + 2 * five / 15 - 17 * seven / 315);
				}
			}

			class SailLead extends Chubgraph
			{
				
				
				SawOsc osc1 => Gain common => LPF lpf => TanDistortion distortion => ADSR a => Gain out => outlet;
				SawOsc osc2 => common;
				SqrOsc osc3 => common;
				0.35 => osc1.gain;
				0.45 => osc2.gain;
				0.20 => osc3.gain;
				1.2 => common.gain; // input to overdrive to distort...
				//1 => overdrive.sync; // SinOsc overdrive
				// dyno.compress();  // Dyno dyno
				13 => lpf.Q;
				0.5 => out.gain;
				staccato( true );    
				
				200 => float goalFreq;
				200 => float currentFreq;
				0.15 => float slewFreq;
				false => int noteIsOn;
				
				fun void staccato( int doStaccato )
				{
					if( doStaccato )
					{
						a.set( 20::ms, 150::ms, 0.3, 1::ms );
					}
					else
					{
						a.set( 20::ms, 5::ms, 0.9, 20::ms );
					}
				}
				
				fun void SetFreq()
				{
					SinOsc pitchLfo => blackhole;
					1.1 => pitchLfo.freq;
					while( true )
					{
						slewFreq * ( goalFreq - currentFreq ) +=> currentFreq;
						// 0.3% of current frequency
						currentFreq + 0.003 * currentFreq * pitchLfo.last() => 
						float waveringFreq;
						// DETUNE
						waveringFreq * detune => waveringFreq; 
						
						
						// BACK TO NORMAL
						waveringFreq => osc1.freq;
						waveringFreq / 2 => osc2.freq;
						waveringFreq => osc3.freq;
						
						// what is 80% of the way up?
						Math.min( waveringFreq * 32, 20000 ) => lpf.freq;
						1::ms => now;
					}
				}
				spork ~ SetFreq();
				
				fun float freq( float f )
				{
					f => goalFreq;
					if( noteIsOn )
					{
						0.026 => slewFreq;
					}
					else
					{
						0.15 => slewFreq;
					}
					
					
					// putting this only here instead of in the while loop
					// makes the attack sound harder because
					// the resonant filter doesn't glide
					//Math.min( goalFreq * 32, 20000 ) => lpf.freq;
					
					return f;
				}
				
				fun float freq()
				{
					return goalFreq;
				}
				
				fun void noteOn()
				{
					if( !noteIsOn )
					{
						1 => a.keyOn;
						goalFreq * 8 => currentFreq; // higher = harder attack
						true => noteIsOn;
					}
					
				}
				
				fun void noteOff()
				{
					1 => a.keyOff;
					false => noteIsOn;
				}
			}

			0.2::second => dur noteLength;


			// sync all rhythmic voices
			fun void Metro()
			{
				while( true )
				{
					metro.broadcast();
					noteLength => now;
				}
			}
			spork ~ Metro();

			[60, 65, 67, 70] @=> int notes[];
			[48, 60, 63, 67, 72] @=> int otherNotes[];
			[
			[60, 65, 67, 70],
			[60, 63, 67, 68],
			[60, 62, 65, 70]
			] @=> int allNotes[][];
			[
			[48, 60, 63, 67, 72],
			[44, 60, 63, 67, 68],
			[46, 60, 62, 65, 70]
			] @=> int allOtherNotes[][];
			0 => int currentChord;

			false => int playPrevNotes;

			fun void Modal()
			{
				int prevNoteIndices[MAX_NOTES_TO_SAVE];
				int prevNoteOctaves[MAX_NOTES_TO_SAVE];
				int prevWasMoreNotes[MAX_NOTES_TO_SAVE];
				int prevNoteWriteIndex; // next place we will write
				int prevNoteReadIndex; // next place we will read
				
				ModalBar bar => Gain g => Pan2 panner;
				2 => g.gain;
				panner.left => left;
				panner.right => right;
				3 => bar.preset;
				
				while( true )
				{
					// COPYPASTA 1
					int note;

					if( playPrevNotes )
					{
						// fetch the note
						notes[ prevNoteIndices[ prevNoteReadIndex ] ] + 12 * prevNoteOctaves[ prevNoteReadIndex ] => note;
						
						// increment
						prevNoteReadIndex++;
						if( prevNoteReadIndex >= NOTES_TO_PLAY ) { 0 => prevNoteReadIndex; }
						
						// prepare over and over again needlessly b/c architecture~~~
						0 => prevNoteWriteIndex;
					}
					else
					{
						// choose and store a new note
						Math.random2( 0, notes.size() - 1 ) => prevNoteIndices[ prevNoteWriteIndex ];
						Math.random2( 0, 2 ) => prevNoteOctaves[ prevNoteWriteIndex ];
						
						// calculate
						notes[ prevNoteIndices[ prevNoteWriteIndex ] ] + 12 * prevNoteOctaves[ prevNoteWriteIndex ] => note;
						
						// store
						false => prevWasMoreNotes[ prevNoteWriteIndex ];
						
						// increment
						prevNoteWriteIndex++;
						if( prevNoteWriteIndex >= MAX_NOTES_TO_SAVE ) { 0 => prevNoteWriteIndex; }
						prevNoteWriteIndex => prevNoteReadIndex;
					}
					
					note => Std.mtof => float f;
					// DETUNE
					f * detune => bar.freq;
					// PLAY 
					Math.random2f( 0.2, 0.8 ) => bar.strikePosition;
					Math.random2f( 0.2, 0.4 ) => bar.strike;
					// PAN
					Math.random2f( -0.8, 0.8 ) => panner.pan;
					
					
					noteLength => dur theNoteLength;
					if( ( playPrevNotes && prevWasMoreNotes[ prevNoteReadIndex ] ) || 
						( PlayMoreNotes() && ( maybe || maybe ) ) )
					{
						0.5::theNoteLength => now;

						// COPYPASTA 2
						if( playPrevNotes )
						{
							// fetch the note
							notes[ prevNoteIndices[ prevNoteReadIndex ] ] + 12 * prevNoteOctaves[ prevNoteReadIndex ] => note;
							
							// increment
							prevNoteReadIndex++;
							if( prevNoteReadIndex >= NOTES_TO_PLAY ) { 0 => prevNoteReadIndex; }
							
							// prepare over and over again needlessly b/c architecture~~~
							0 => prevNoteWriteIndex;
						}
						else
						{
							// choose and store a new note
							Math.random2( 0, notes.size() - 1 ) => prevNoteIndices[ prevNoteWriteIndex ];
							Math.random2( 0, 2 ) => prevNoteOctaves[ prevNoteWriteIndex ];
							
							// calculate
							notes[ prevNoteIndices[ prevNoteWriteIndex ] ] + 12 * prevNoteOctaves[ prevNoteWriteIndex ] => note;
							
							// store
							true => prevWasMoreNotes[ prevNoteWriteIndex ];
							
							// increment
							prevNoteWriteIndex++;
							if( prevNoteWriteIndex >= MAX_NOTES_TO_SAVE ) { 0 => prevNoteWriteIndex; }
							prevNoteWriteIndex => prevNoteReadIndex;
						}
						
						note => Std.mtof => float f;
						// DETUNE
						f * detune => bar.freq;
						// PLAY 
						Math.random2f( 0.2, 0.8 ) => bar.strikePosition;
						Math.random2f( 0.2, 0.4 ) => bar.strike;
						// PAN
						Math.random2f( -0.8, 0.8 ) => panner.pan;
						
						// don't do, instead use metro
						// 0.5::theNoteLength => now;
					}
					/*else
					{
						theNoteLength => now;
					}*/
					metro => now;
				}
				
			}
			spork ~ Modal();

			fun void Sail()
			{
				int prevNoteIndices[MAX_NOTES_TO_SAVE];
				int prevNoteOctaves[MAX_NOTES_TO_SAVE];
				int prevShouldTriplePlay[MAX_NOTES_TO_SAVE];
				int prevShouldDoublePlay[MAX_NOTES_TO_SAVE];
				int prevNoteWriteIndex; // next place we will write
				int prevNoteReadIndex; // next place we will read
				
				SailLead bass => LPF bassLPF => Pan2 panner;
				panner.left => left;
				panner.right => right;
				
				while( true )
				{
					noteLength => dur theFirstNoteLength;
					
					// COPYPASTA 3
					int note;
					false => int shouldDoublePlay;
					false => int shouldTriplePlay;
					
					if( playPrevNotes )
					{
						// fetch the note
						notes[ prevNoteIndices[ prevNoteReadIndex ] ] + 12 * prevNoteOctaves[ prevNoteReadIndex ] => note;
						prevShouldDoublePlay[ prevNoteReadIndex ] => shouldDoublePlay;
						
						// increment
						prevNoteReadIndex++;
						if( prevNoteReadIndex >= NOTES_TO_PLAY ) { 0 => prevNoteReadIndex; }
						
						// prepare over and over again needlessly b/c architecture~~~
						0 => prevNoteWriteIndex;
					}
					else
					{
						// choose and store a new note
						Math.random2( 0, notes.size() - 1 ) => prevNoteIndices[ prevNoteWriteIndex ];
						Math.random2( -1, 1 ) => prevNoteOctaves[ prevNoteWriteIndex ];
						
						// choose
						PlayMoreNotes() && maybe => shouldDoublePlay;
						shouldDoublePlay && maybe => shouldTriplePlay;  

						// store
						shouldDoublePlay => prevShouldDoublePlay[prevNoteWriteIndex];
						shouldTriplePlay => prevShouldTriplePlay[prevNoteWriteIndex];
						
						// calculate
						notes[ prevNoteIndices[ prevNoteWriteIndex ] ] + 12 * prevNoteOctaves[ prevNoteWriteIndex ] => note;
									
						// increment
						prevNoteWriteIndex++;
						if( prevNoteWriteIndex >= MAX_NOTES_TO_SAVE ) { 0 => prevNoteWriteIndex; }
						prevNoteWriteIndex => prevNoteReadIndex;
					}

					
					
					note => Std.mtof => bass.freq;
					bass.freq() * 1.5 => bassLPF.freq;
					bass.noteOn();
					
					if( shouldDoublePlay )
					{            
						0.3::theFirstNoteLength => now;
						bass.noteOff();
						0.2::theFirstNoteLength => now;
						
						bass.freq() * 0.5 => bass.freq;
						bass.noteOn();
						
						// don't do, instead use metro
						// 0.5::theFirstNoteLength => now;
					}
					/*
					else
					{
						theFirstNoteLength => now;
					}*/
					metro => now;
					
					bass.noteOff();
					
					noteLength => dur theSecondNoteLength;
					
					if( shouldTriplePlay )
					{
						// COPYPASTA 4
						if( playPrevNotes )
						{
							// fetch the note
							notes[ prevNoteIndices[ prevNoteReadIndex ] ] + 12 * prevNoteOctaves[ prevNoteReadIndex ] => note;
							
							// increment
							prevNoteReadIndex++;
							if( prevNoteReadIndex >= NOTES_TO_PLAY ) { 0 => prevNoteReadIndex; }
							
							// prepare over and over again needlessly b/c architecture~~~
							0 => prevNoteWriteIndex;
						}
						else
						{
							// choose and store a new note
							Math.random2( 0, notes.size() - 1 ) => prevNoteIndices[ prevNoteWriteIndex ];
							Math.random2( -1, 1 ) => prevNoteOctaves[ prevNoteWriteIndex ];
							
							// store
							false => prevShouldDoublePlay[prevNoteWriteIndex];
							false => prevShouldTriplePlay[prevNoteWriteIndex];
							
							// calculate
							notes[ prevNoteIndices[ prevNoteWriteIndex ] ] + 12 * prevNoteOctaves[ prevNoteWriteIndex ] => note;
							
							// increment
							prevNoteWriteIndex++;
							if( prevNoteWriteIndex >= MAX_NOTES_TO_SAVE ) { 0 => prevNoteWriteIndex; }
							prevNoteWriteIndex => prevNoteReadIndex;
						}
						

						note => Std.mtof => bass.freq;
						bass.freq() * 1.5 => bassLPF.freq;
						bass.noteOn();

						0.8::theSecondNoteLength => now;
						
						bass.noteOff();
						// don't do, instead do metro
						// 0.2::theSecondNoteLength => now;
					}
					/*
					else
					{
						theSecondNoteLength => now;
					}
					*/
					
					metro => now;
				}    
			}
			spork ~ Sail();


			fun void Ahh()
			{
				AhhSynth synths[otherNotes.size()];
				Pan2 panners[otherNotes.size()];
				[0., -0.7, -0.3, 0.7, 0.3] @=> float pans[];
				
				0.1 => float noteSlew;
				float currentNotes[otherNotes.size()];
				
				for( int i; i < synths.size(); i++ )
				{
					otherNotes[i] => currentNotes[i];
					panners[i].left => left;
					panners[i].right => right;
					pans[i] => panners[i].pan;

					1.0 / synths.size() => synths[i].gain;
					synths[i] => panners[i];
				}
				
				while( true )
				{
					for( int i; i < otherNotes.size(); i++ )
					{
						noteSlew * ( otherNotes[i] - currentNotes[i] ) +=> currentNotes[i];
						currentNotes[i] => Std.mtof => float f;
						// DETUNE
						f * detune => synths[i].freq;
					}
					10::ms => now;
				}
			}
			spork ~ Ahh();

			fun void ReactToSpaceBar()
			{
				// wait
				metro => now;
				
				// increment current chord
				currentChord++;
				if( currentChord >= allNotes.size() )
				{
					0 => currentChord;
				}
				
				for( int i; i < allNotes[currentChord].size(); i++ )
				{
					allNotes[currentChord][i] => notes[i];
				}
				for( int i; i < allOtherNotes[currentChord].size(); i++ )
				{
					allOtherNotes[currentChord][i] => otherNotes[i];
				}
			}

			float frozenActivation;
			0.1 => float frozenSlew;

			fun void FreezeNotes()
			{
				// signal to start looping notes
				true => playPrevNotes;
				
				// freeze tempo and detune
				0 => frozenSlew;
				
				// print
				<<< ""Frozen at"", noteLength / second, ""seconds / note and detune"", detune >>>;
			}

			fun void UnFreezeNotes()
			{
				// signal to stop looping notes
				false => playPrevNotes;
				
				// slowly unfreeze tempo and detune
				0.1 => frozenSlew;
				
				// print
				<<< ""Unfrozen"", """" >>>;
			}

			fun void ReactToShift( int onBool )
			{
				if( onBool )
				{
					FreezeNotes();
				}
				else
				{
					UnFreezeNotes();
				}
			}


			fun void ReactToNumber( int n )
			{
				// set the loop length to be n+3
				n + 3 => NOTES_TO_PLAY;
				<<< ""Loop is now"", NOTES_TO_PLAY, ""notes"" >>>;
			}

			Event endPiece;
			fun void Keyb()
			{
				Hid hi;
				HidMsg msg;
				
				if( !hi.openKeyboard( 0 ) ) me.exit();
				
				// time-loop
				// infinite event loop
				while( true )
				{
					// wait for event
					hi => now;
					
					// get message
					while( hi.recv( msg ) )
					{
						// check
						if( msg.which == 42 || msg.which == 54 || msg.which == 225 || msg.which == 229 ) // on windows: 42, 54 // on mac: 225, 229
						{
							ReactToShift( msg.isButtonDown() );
						}
						else if( msg.isButtonDown() )
						{
							if( msg.which == 57 || msg.which == 44 ) // on windows: 57 // on mac: 44
							{
								// SPACE BAAAAR
								ReactToSpaceBar();
							}
							else if( msg.which >= 2 && msg.which < 12 ) // on windows: [2, 12) // on mac: [30, 40)
							{
								ReactToNumber( msg.which - 1 ); // on windows: -1 // on mac: -29
							}
							else if( msg.which >= 30 && msg.which < 40 ) // on windows: [2, 12) // on mac: [30, 40)
							{
								ReactToNumber( msg.which - 29 ); // on windows: -1 // on mac: -29
							}
							else if( msg.which == 28 || msg.which == 40 ) // on windows: 28 // on mac: 40
							{
								// enter key: end piece
								endPiece.broadcast();
							}                
						}
					}
				}
			}
			spork ~ Keyb();


			fun void MeasureMicLoudness()
			{
				float theLoudness;
				// patch
				adc => Gain g => OnePole p => blackhole;
				// square the input
				adc => g;
				// multiply
				3 => g.op;
				
				// set pole position
				0.99 => p.pole;

				// fast reduction: 0.9 or less (for 100::ms)
				// slow reduction: 0.999
				// medium reduction: 0.97
				// for 10::ms:
				0.996 => float activationReduction;
				
				while( true )
				{
					// compute loudness
					if( p.last() < 0.000012 )
					{
						0 => theLoudness;
					}
					else
					{
						( p.last() - 0.000012 ) / 0.25 => theLoudness;
					}
					
					// maximum value is 0.1
					if( theLoudness > 0.1 ) { 0.1 => theLoudness; }
					
					// compute activation
					theSumActivation * activationReduction 
					+ theLoudness * 0.2 => theSumActivation;
					
					if( theSumActivation > 1 ) { 1 => theSumActivation; }
					
					// test value
					10::ms => now;
				}
			}
			// don't use mic in unity
			//spork ~ MeasureMicLoudness();

			fun int PlayMoreNotes()
			{
				return frozenActivation > 0.5;
			}

			fun void AssignThingsFromMicLoudness()
			{
				float smoothedActivation;
				0.1 => float smoothedSlew;
				while( true )
				{ 
					// smooth
					smoothedSlew * ( theSumActivation - smoothedActivation ) +=> smoothedActivation;
					frozenSlew * ( theSumActivation - frozenActivation ) +=> frozenActivation;
					
					// set detune
					0.7 => float detuneCutoff;
					0.943875 => float detuneAmount; // 1/2 step lower
					0.5 => detuneAmount; // octave lower
					0.98 => detuneAmount; // only a little lower
					0.97 => detuneAmount; // a little lower
					0.96 => detuneAmount; // even lower
					
					// set detune
					if( smoothedActivation < detuneCutoff )
					{
						// frozen: freezes when hit shift
						Std.scalef( frozenActivation, 0, detuneCutoff, detuneAmount, 1 ) => detune;
					}
					else
					{
						1 => detune;
					}
					
					// set tempo (frozen: freezes when hit shift)
					Std.scalef( frozenActivation, 0, 1, 0.8, 0.2 )::second => noteLength;
					10::ms => now;
					
					// set LPF cutoff (does not freeze when hit shift)
					Std.scalef( smoothedActivation, 0, 1, 50, 135 ) => Std.mtof => lLPF.freq => rLPF.freq;
					
				}
			}
			spork ~ AssignThingsFromMicLoudness();

			// dac => WvOut2 wavOut => blackhole;
			// me.dir() + ""performance.wav"" => wavOut.wavFilename;
			// 2::minute => now;
			// wavOut.closeFile();

			endPiece => now;

			while( true ) 
			{
				// turn everything off...
				0.99 * left.gain() => left.gain => right.gain;
				10::ms => now;

			}

			while( true ) 
			{ 
				0.25::second => now; 
				// test value
				chout <= theSumActivation <= ""     "" <= mouseActivation <= IO.newline();
					
			}
		
		");
	}
}
